/*
Save and restore VBE state 

Written by Matthew Garrett <mjg59@srcf.ucam.org>, heavily based on
vbetest.c from the lrmi package

This program is in the public domain. No rights claimed.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/io.h>
#include <sys/kd.h>
#include <sys/stat.h>

#include "lrmi.h"

void
save_state()
{
	struct LRMI_regs r;
	void *buffer;
	unsigned int size;

	memset(&r, 0, sizeof(r));

	r.eax = 0x4f04;
	r.ecx = 0xf; 	/* all states */
	r.edx = 0; 	/* get buffer size */

	if (!LRMI_int(0x10, &r)) {
		fprintf(stderr, "Can't get video state buffer size (vm86 failure)\n");
	}

	if ((r.eax & 0xffff) != 0x4f) {
		fprintf(stderr, "Get video state buffer size failed\n");
	}
	
	size = (r.ebx & 0xffff) * 64;

	buffer = LRMI_alloc_real(size);

	if (buffer == NULL) {
		fprintf(stderr, "Can't allocate video state buffer\n");
	}

	memset(&r, 0, sizeof(r));

	r.eax = 0x4f04;
	r.ecx = 0xf; 	/* all states */
	r.edx = 1; 	/* save state */
	r.es = (unsigned int)buffer >> 4;
	r.ebx = (unsigned int)buffer & 0xf;

	if (!LRMI_int(0x10, &r)) {
		fprintf(stderr, "Can't save video state (vm86 failure)\n");
	}

	if ((r.eax & 0xffff) != 0x4f) {
		fprintf(stderr, "Save video state failed\n");
	}

	write (1,buffer,size);
}

void
restore_state(void *buffer)
{
	struct LRMI_regs r;

	memset(&r, 0, sizeof(r));

	r.eax = 0x4f04;
	r.ecx = 0xf; 	/* all states */
	r.edx = 2; 	/* restore state */
	r.es = (unsigned int)buffer >> 4;
	r.ebx = (unsigned int)buffer & 0xf;

	if (!LRMI_int(0x10, &r)) {
		fprintf(stderr, "Can't restore video state (vm86 failure)\n");
	} else if ((r.eax & 0xffff) != 0x4f) {
		fprintf(stderr, "Restore video state failed\n");
	}

	LRMI_free_real(buffer);
}

void
text_mode(void)
{
	struct LRMI_regs r;

	memset(&r, 0, sizeof(r));

	r.eax = 3;

	if (!LRMI_int(0x10, &r)) {
		fprintf(stderr, "Can't set text mode (vm86 failure)\n");
	}
}

int
main(int argc, char *argv[])
{
	if (!LRMI_init())
		return 1;

	ioperm(0, 1024, 1);
	iopl(3);


	if (!strcmp(argv[1],"save")) {
		save_state();
	} else if (!strcmp(argv[1],"restore")) {
		void *data = NULL;
		short tmpbuffer[524288];
		size_t i,length=0;
		
		while (i=read(0, tmpbuffer, sizeof(tmpbuffer))) {
			length+=i;
		}

		data = LRMI_alloc_real(length);
		memcpy(data,tmpbuffer,length);

		restore_state(data);
	} else {
		fprintf(stderr,"%s: Usage %s [save|restore]\n",
			argv[0],argv[0]);
	}
	       

	return 0;
}
